﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Windows.Controls.Primitives;
using System.Windows.Media.Animation;
using APlan.Service;
using APlan.ViewModel;
namespace APlan.Client
{
    /// <summary>
    /// Interaction logic for AssignmentView.xaml
    /// </summary>
    public partial class AssignmentView : UserControl
    {
            
        public static AssignmentView instance = new AssignmentView();
        
        static ZoomableCanvas AssignmentCanvas, EmpCanvas, TimelineCanvas;
        Point LastMousePosition;
        private int drag;
        private Point origPos;
        private Point? zoomTargetPos, zoomTimelineTargetPos, zoomEmpTargetPos;
        private double zoomScaleFact;
        private AssignmentViewModel draggedAssignment;
        private double scaleTo;        

        public AssignmentView()
        {
            InitializeComponent();
            GalaSoft.MvvmLight.Messaging.Messenger.Default.Register<AssignmentViewModel>(this, OnZoomMessage);
            GalaSoft.MvvmLight.Messaging.Messenger.Default.Register<Vector>(this, onPanMessage);
        }

        /// <summary>
        /// update the offset after paning: when changing the offset with an animation, this has to be done
        /// </summary>
        /// <param name="val"></param>
        private void updatePan(Vector val)
        {
            AssignmentCanvas.Offset += val;
            
            Vector timeval = new Vector(val.X, 0);
            TimelineCanvas.Offset += timeval;
            Vector empval = new Vector(0, val.Y);
            EmpCanvas.Offset += empval;

            if (AssignmentCanvas.Offset.Y < 0)
            {
                Vector zero = new Vector(0, AssignmentCanvas.Offset.Y * (-1));
                AssignmentCanvas.Offset += zero;
                EmpCanvas.Offset += zero;
            }
        }

        private void onPanMessage(Vector val)
        {
            AssignmentCanvas.Dispatcher.BeginInvoke(new Action<Vector>(updatePan), val);
        }
        private void OnZoomMessage(AssignmentViewModel a)
        {
            if (AssignmentCanvas.ActualViewbox.Top < a.Y &&
                AssignmentCanvas.ActualViewbox.Left < a.X &&
                AssignmentCanvas.ActualViewbox.Right > a.X + a.Width &&
                AssignmentCanvas.ActualViewbox.Bottom > a.Y + a.Height)
            {

            }
            else
            {
                TimeSpan duration = new TimeSpan(0, 0, 0, 4);
                TimeSpan durscal = new TimeSpan(0,0,0,2);
                // the assignment is outside of the actual viewbox -> zoom to the assignment
                Point targetpos = new Point();
                /* with scaling?
                targetpos.X = a.X - AssignmentCanvas.ActualViewbox.Width / 2;
                targetpos.Y = a.Y - AssignmentCanvas.ActualViewbox.Width / 2;
                 * */
                targetpos.X = a.X - AssignmentCanvas.ActualViewbox.Width / 2; ;
                targetpos.Y = a.Y - AssignmentCanvas.ActualViewbox.Height / 2;
                if (targetpos.Y < 0)
                    targetpos.Y = 0;
                zoomTargetPos = targetpos;
                PointAnimation anim = new PointAnimation();
                anim.From = AssignmentCanvas.Offset;
                anim.To = targetpos;
                anim.AccelerationRatio = 0.25;
                anim.DecelerationRatio = 0.25;
                anim.Duration = duration;
                LastMousePosition = targetpos;
                AssignmentCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, anim);

                Point targetTimeline = new Point(targetpos.X, 0);
                PointAnimation animTimeline = new PointAnimation();
                animTimeline.From = TimelineCanvas.Offset;
                animTimeline.To = targetTimeline;
                animTimeline.DecelerationRatio = 0.25;
                animTimeline.AccelerationRatio = 0.25;
                animTimeline.Duration = duration;
                zoomTimelineTargetPos = targetTimeline;
                TimelineCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, animTimeline);

                Point targetEmp = new Point(0, targetpos.Y);
                PointAnimation animEmp = new PointAnimation();
                animEmp.From = EmpCanvas.Offset;
                animEmp.To = targetEmp;
                animEmp.DecelerationRatio = 0.25;
                animEmp.AccelerationRatio = 0.25;
                animEmp.Duration = duration;
                zoomEmpTargetPos = targetEmp;
                EmpCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, animEmp);

                double fact = Math.Sqrt( Math.Pow(AssignmentCanvas.Offset.X - targetpos.X,2) + Math.Pow( AssignmentCanvas.Offset.Y - targetpos.Y, 2) ) / AssignmentCanvas.ActualViewbox.Width;
                if (fact > 1)
                {
                    fact = Math.Pow( Math.Pow(2, -fact), 0.1);

                    zoomScaleFact = fact;

                    DoubleAnimation scaleStart = new DoubleAnimation();
                    scaleStart.From = AssignmentCanvas.Scale;
                    scaleStart.To = AssignmentCanvas.Scale * zoomScaleFact;
                    scaleStart.Duration = durscal;
                    scaleStart.AccelerationRatio = 0.8;
                    scaleStart.Completed += new EventHandler(scaleStart_Completed);
                    /*
                    AssignmentCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleStart);
                    TimelineCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleStart);
                    EmpCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleStart);
                     * */
                }
            }
        }

        void scaleStart_Completed(object sender, EventArgs e)
        {
            if (1 == 1)
            {
                AnimationClock c = (AnimationClock)sender;
                double s = (double) c.Timeline.GetAnimationBaseValue(ZoomableCanvas.ScaleProperty);
                DoubleAnimation scaleEnd = new DoubleAnimation();
                scaleEnd.From = s * zoomScaleFact;
                scaleEnd.To = s;
                scaleEnd.Duration = new TimeSpan(0, 0, 1);
                scaleEnd.DecelerationRatio = 0.5;
                AssignmentCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleEnd);
                TimelineCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleEnd);
                EmpCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, scaleEnd);
            }
        }
        private void ZoomableCanvas_Loaded(object sender, RoutedEventArgs e)
        {
            // Store the canvas in a local variable since x:Name doesn't work.
            AssignmentCanvas = (ZoomableCanvas)sender;
            scaleTo = AssignmentCanvas.Scale;
        }
        private void ZoomableCanvasTimeline_Loaded(object sender, RoutedEventArgs e)
        {
            // Store the canvas in a local variable since x:Name doesn't work.
            TimelineCanvas = (ZoomableCanvas)sender;
        }
        private void ZoomableCanvasEmp_Loaded(object sender, RoutedEventArgs e)
        {
            EmpCanvas = (ZoomableCanvas)sender;
        }

        /// <summary>
        /// check if the MouseMove should do paning or dragging
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPreviewMouseMove(MouseEventArgs e)
        {
            var position = e.GetPosition(AssignmentsList);
            if (e.LeftButton == MouseButtonState.Pressed
                && !(e.OriginalSource is Thumb || e.OriginalSource is Button)) // Don't block the scrollbars.
            {
                if (drag != 1)
                {
                    AssignmentCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
                    TimelineCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
                    EmpCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
                    if (zoomTargetPos != null)
                    {
                        // the offset is not set, when a previous animated panning happened,
                        // so the offset is set manually here...
                        AssignmentCanvas.Offset = (Point)zoomTargetPos;
                        TimelineCanvas.Offset = (Point)zoomTimelineTargetPos;
                        EmpCanvas.Offset = (Point)zoomEmpTargetPos;
                        zoomTargetPos = null;
                        zoomEmpTargetPos = null;
                        zoomTimelineTargetPos = null;
                    }
                    CaptureMouse();
                    Vector newpos = new Vector();
                    newpos = position - LastMousePosition;

                    if (AssignmentCanvas.Offset.Y - newpos.Y < 0)
                    {
                        newpos.Y = 0;
                    }
                              
                    AssignmentCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
                    AssignmentCanvas.Offset -= newpos;                    

                    Vector timelinepos = new Vector(newpos.X, 0);                    
                    TimelineCanvas.Offset -= timelinepos;
                    Vector Empnewpos = new Vector(0, newpos.Y);                    
                    EmpCanvas.Offset -= Empnewpos;
                    e.Handled = true;
                }
                else
                {
                    Point curPos = e.GetPosition(AssignmentCanvas);
                    Vector diff = curPos - origPos;
                    origPos = curPos;
                    
                    draggedAssignment.IsDragging = true;
                    draggedAssignment.X += diff.X;
                    draggedAssignment.Y += diff.Y;
                    
                    e.Handled = true;                    
                }
            }
            else
            {
                ReleaseMouseCapture();
            }
            LastMousePosition = position;
        }

        /// <summary>
        /// start zooming
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPreviewMouseWheel(MouseWheelEventArgs e)
        {
            const int animDuration = 250;
            var x = Math.Pow(2, e.Delta / 4.0 / Mouse.MouseWheelDeltaForOneLine);
            DoubleAnimation da = new DoubleAnimation();
            da.From = AssignmentCanvas.Scale;
            da.To = AssignmentCanvas.Scale * x;
            scaleTo = (double)da.To;
            da.Duration = new TimeSpan(0, 0, 0, 0, animDuration);
            da.Completed += new EventHandler(MouseWheelScale_Completed);
            /*
            da.AccelerationRatio = 0.3;
            da.DecelerationRatio = 0.3;
             */
            AssignmentCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, da);
            EmpCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, da);
            TimelineCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, da);
            
            // Adjust the offset to make the point under the mouse stay still.
            Point mousepos = e.GetPosition(AssignmentsList);
            LastMousePosition = mousepos;
            var position = (Vector)mousepos;
            
            Vector newposdiff = ((Vector)(AssignmentCanvas.Offset + position) * x - position);
            Point newpos = new Point(newposdiff.X, newposdiff.Y);
            if (newpos.Y < 0)
            {
                newpos.Y = 0;
            }
            zoomTargetPos = newpos;
            PointAnimation assignOffsetAnim = new PointAnimation();
            assignOffsetAnim.From = AssignmentCanvas.Offset;
            assignOffsetAnim.To = newpos;
            assignOffsetAnim.Duration = new TimeSpan(0, 0, 0, 0, animDuration);
            /*assignOffsetAnim.DecelerationRatio = 0.3;
            assignOffsetAnim.AccelerationRatio = 0.3;*/
            //assignOffsetAnim.FillBehavior = FillBehavior.Stop;
            //MyCanvas.Offset = newpos;
            AssignmentCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, assignOffsetAnim);

            // The Timeline Canvas has the same offset, but y is always 0             
            PointAnimation timelineOffsetAnim = new PointAnimation();
            timelineOffsetAnim.From = TimelineCanvas.Offset;
            timelineOffsetAnim.To = new Point(newpos.X,0);
            zoomTimelineTargetPos = timelineOffsetAnim.To;
            timelineOffsetAnim.Duration = new TimeSpan(0, 0, 0, 0, animDuration);
            /*timelineOffsetAnim.AccelerationRatio = 0.3;
            timelineOffsetAnim.DecelerationRatio = 0.3;*/
            TimelineCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, timelineOffsetAnim);
            //TimelineCanvas.Offset = timelinepos;
            
            // The Employee Canvas has the same offset, but x is always 0 
            PointAnimation empOffsetAnim = new PointAnimation();
            empOffsetAnim.From = EmpCanvas.Offset;
            empOffsetAnim.To = new Point(0, newpos.Y);
            zoomEmpTargetPos = empOffsetAnim.To;
            empOffsetAnim.Duration = timelineOffsetAnim.Duration;
            /*empOffsetAnim.DecelerationRatio = 0.3;
            empOffsetAnim.AccelerationRatio = 0.3;*/
            EmpCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, empOffsetAnim);
             
            e.Handled = true;
        }

        /// <summary>
        /// finish the zoom animation of mousewheel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MouseWheelScale_Completed(object sender, EventArgs e)
        {
            AssignmentCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
            TimelineCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
            EmpCanvas.BeginAnimation(ZoomableCanvas.OffsetProperty, null);
            AssignmentCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, null);
            TimelineCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, null);
            EmpCanvas.BeginAnimation(ZoomableCanvas.ScaleProperty, null);

            ViewModelLocator.MainStatic.CanvasScaleFactor = scaleTo;
            if (zoomTargetPos != null)
                AssignmentCanvas.Offset = (Point)zoomTargetPos;
            if (zoomTimelineTargetPos != null)
                TimelineCanvas.Offset = (Point)zoomTimelineTargetPos;
            if(zoomEmpTargetPos != null)
                EmpCanvas.Offset = (Point)zoomEmpTargetPos;
        }

        private void Element_MouseDown(object sender, MouseButtonEventArgs e)
        {
            
            Border grid = (Border)sender;
            AssignmentViewModel av = (AssignmentViewModel)grid.DataContext;
            if (av.Type != (int)AssignmentType.bar)
            {
                origPos = e.GetPosition(AssignmentCanvas);
                drag = 1;

                draggedAssignment = (AssignmentViewModel)grid.DataContext;

                GalaSoft.MvvmLight.Messaging.Messenger.Default.Send<AssignmentViewModel>(draggedAssignment);
                e.Handled = true;
            }
        }

        protected override void OnPreviewMouseUp(MouseButtonEventArgs e)
        {
            if (drag == 1)
            {
                draggedAssignment.SetToNextEmployee();
                draggedAssignment.RoundMinutes();
                draggedAssignment.Opacity = 1;
                draggedAssignment.IsDragging = false;
                drag = 0;
                draggedAssignment = null;
                e.Handled = true;
                
            }
            base.OnPreviewMouseUp(e);
        }

        /// <summary>
        /// check if an assignment has been droped on the AssignmentView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AssignmentsList_Drop(object sender, DragEventArgs e)
        {
            CustIBaseViewModel ci;
            if (e.Data.GetDataPresent("failure") || e.Data.GetDataPresent("assignment"))
            {
                Point pos = e.GetPosition((IInputElement) sender);
                AssignmentViewModel av = new AssignmentViewModel(ViewModelLocator.MainStatic);
                if (e.Data.GetDataPresent("failure"))
                {
                    ci = e.Data.GetData("failure") as CustIBaseViewModel;
                    av.Type = (short)APlan.Service.AssignmentType.failure;
                    ViewModelLocator.MainStatic.RemoveOpenFailure(ci.Model.Ibase);
                }
                else
                {
                    ci = e.Data.GetData("assignment") as CustIBaseViewModel;
                    av.Type = (short)APlan.Service.AssignmentType.maintenance;
                }
                
                av.Model.CCity = ci.Model.City;
                av.Model.CHNr = ci.Model.HNr;
                av.Model.City = ci.Model.IBaseCity;               
                av.Model.CPost_code = ci.Model.Post_code;
                av.Model.CStreet = ci.Model.Street;
                av.Model.fix = false;
                av.Model.HNr = ci.Model.IBaseHNr;
                av.Model.Ibase = ci.Model.Ibase;
                av.Model.Info = ci.Model.Info;
                av.IsSelected = true;
                av.Model.Name = ci.Model.Name;
                av.Model.Phone = ci.Model.Phone;
                av.Model.Post_code = ci.Model.IBasePostCode;
                av.State = APlan.Service.AssignmentState.inform;
                av.Model.Street = ci.Model.IBaseStreet;                
                
                av.Duration = ViewModelLocator.MainStatic.DefaultAssignmentDuration;
                av.X = AssignmentCanvas.Offset.X + pos.X - (av.Width / 2);
                av.Y = AssignmentCanvas.Offset.Y + pos.Y - (av.Height / 2);
                av.To = av.From.AddMinutes(av.Duration);
                av.Height = ViewModelLocator.MainStatic.ElementHeight;
                av.SetToNextEmployee();
                
                ViewModelLocator.MainStatic.InsertDragDropAssignment(av.GetAssignmentObject());
                //ViewModelLocator.MainStatic.Assignments.Add(av);
            }

        }
              
    }
}
