﻿using System.Windows;

using System.Collections.ObjectModel;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using APlan.ViewModel;
using APlan.Service;
using System.Threading;
using GMap.NET;
using GMap.NET.WindowsPresentation;

namespace APlan.Client
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {        
        AssignmentView assignmentControl;
        Point startpoint;
        Vector panvect;
        SuggestAssignmentWindow assignmentWindow=null;
        bool pan = false;
        public MainWindow()
        {
            InitializeComponent();
            Closing += (s, e) => ViewModelLocator.Cleanup();
            startpoint = new Point();

            //init GMap
            System.Net.IPHostEntry entr = System.Net.Dns.GetHostEntry("www.google.com");
            GMapControl.Manager.Mode = AccessMode.ServerAndCache;
            GMaps.Instance.Mode = AccessMode.ServerAndCache;
        }

        private void AssignmentViewControl_Loaded(object sender, RoutedEventArgs e)
        {
            assignmentControl = (AssignmentView)sender;
            ViewModelLocator.MainStatic.LoadData();
        }        

        private void Window_Closed(object sender, System.EventArgs e)
        {
            ViewModelLocator.MainStatic.Close();
        }

        private void FailureMockup_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            startpoint = e.GetPosition(null);
            e.Handled = true;
        }

        /// <summary>
        /// when the mouse is clicked an moved over the FailureMockup start drag
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FailureMockup_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            Point mousepos = e.GetPosition(null);
            Vector diff = startpoint - mousepos;
            if (e.LeftButton == MouseButtonState.Pressed &&
                System.Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance &&
                System.Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance)
            {
                if (ViewModelLocator.MainStatic.OpenFailureCollectionView.CurrentItem != null)
                {
                    Border failure = FindAnchestor<Border>((DependencyObject)e.OriginalSource);
                    DataObject dragdata = new DataObject("failure", ViewModelLocator.MainStatic.OpenFailureCollectionView.CurrentItem);
                    DragDrop.DoDragDrop(failure, dragdata, DragDropEffects.Move);
                }
            }
        }

        // Helper to search up the VisualTree
        private static T FindAnchestor<T>(DependencyObject current)
            where T : DependencyObject
        {
            do
            {
                if (current is T)
                {
                    return (T)current;
                }
                current = VisualTreeHelper.GetParent(current);
            }
            while (current != null);
            return null;
        }
        /// <summary>
        /// when the mouse is clicked an moved over the AssignmentMockup start drag
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Border_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            Point mousepos = e.GetPosition(null);
            Vector diff = startpoint - mousepos;
            if (e.LeftButton == MouseButtonState.Pressed &&
                System.Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance &&
                System.Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance)
            {
                if (ViewModelLocator.MainStatic.CustomerCollectionView.CurrentItem != null)
                {
                    Border assignment = FindAnchestor<Border>((DependencyObject)e.OriginalSource);
                    DataObject data = new DataObject("assignment", ViewModelLocator.MainStatic.CustomerCollectionView.CurrentItem);
                    DragDrop.DoDragDrop(assignment, data, DragDropEffects.Move);
                }
            }
        }

        /// <summary>
        /// start paning when the navigation control is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ellipse_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ThreadStart pandelegate = new ThreadStart(this.doPanning);
            Thread thread = new Thread(pandelegate);

            IInputElement help = (IInputElement)sender;
            Point panpoint = e.GetPosition(help);
            setPaning(sender, panpoint);

            thread.Start();
        }

        /// <summary>
        /// change the paning direction, when the mouse moves on the navigation control
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ellipse_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                IInputElement help = (IInputElement)sender;
                Point panpoint = e.GetPosition(help);
                setPaning(sender, panpoint);
            }
        }

        /// <summary>
        /// update paning information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="panpoint"></param>
        private void setPaning(object sender, Point panpoint)
        {
            Ellipse ellipse = (Ellipse)sender;
            double ellWidth = ellipse.RenderedGeometry.Bounds.Width / 2;
            double ellHeight = ellipse.RenderedGeometry.Bounds.Height / 2;
            panvect.X = panpoint.X - ellWidth;
            panvect.Y = panpoint.Y - ellHeight;
            pan = true;
        }

        /// <summary>
        /// stop paning when mouse is released
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ellipse_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            pan = false;
        }

        private void doPanning()
        {
            while (pan)
            {
                GalaSoft.MvvmLight.Messaging.Messenger.Default.Send<Vector>(panvect);
                Thread.Sleep(70);
            }
            if (1 == 1)
            {
            }
        }

        private void btnZoomOut_Click(object sender, RoutedEventArgs e)
        {
            ZoomSlider.Value -= 0.1;
        }

        private void btnZoomIn_Click(object sender, RoutedEventArgs e)
        {
            ZoomSlider.Value += 0.1;
        }

        private void btnSuggestAssignment_Click(object sender, RoutedEventArgs e)
        {
            if (assignmentWindow == null)
            {
                assignmentWindow = new SuggestAssignmentWindow();
                assignmentWindow.Show();
                assignmentWindow.Closed += new System.EventHandler(assignmentWindow_Closed);
            }
            
        }

        void assignmentWindow_Closed(object sender, System.EventArgs e)
        {
            assignmentWindow = null;
        }

        /// <summary>
        /// update the map when a new customer is selected
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void IBaseAdress_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            CustIBaseViewModel cust = (CustIBaseViewModel)e.NewValue;
            GeoCoderStatusCode ret = GMapControl.SetCurrentPositionByKeywords(cust.IBaseAdress);
        }

        
        
    }

}
