﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using APlan.Service;
using System.ComponentModel;
using System.Windows.Media;
using System.ServiceModel;
using System.Collections.ObjectModel;
using APlan;
using GalaSoft.MvvmLight;

namespace APlan.ViewModel
{
    /// <summary>
    /// The class AssignmentViewModel is the Viewmodel für an assignment. 
    /// </summary>
    public class AssignmentViewModel : ViewModelBase
    {
        
        //reference to the main view model
        private MainViewModel src;        

        //data of an assignment
        private double x, y, duration, width, height, _opacity;
        //private DateTime from, to;

        //the correspodning employee
        private EmployeeViewmodel employee;

        //isDragging is true while the user drag the assignment
        private Boolean isDragging;

        //isSelected is true, when the assignment is selected
        private Boolean isSelected;

        //state of the assignment, for example open
        private AssignmentState state;

        private int fontsize;

        public AssignmentIBaseView Model
        {
            get;
            set;
        }

        public AssignmentViewModel(MainViewModel a)
        {            
            src = a;
            Type = (short) AssignmentType.maintenance;
            State = AssignmentState.open;
            Opacity = 1;
            fontsize = 11;
            Model = new AssignmentIBaseView();
        }
        public AssignmentViewModel(MainViewModel main, AssignmentIBaseView a): this(main)
        {
            this.From = (DateTime)a.FromDate;
            this.To = (DateTime)a.ToDate;
            this.Employee = ViewModelLocator.MainStatic.Employeedict[a.Employee];            
            this.State = (AssignmentState)a.State;
            this.Type = (short)a.Type;
            this.Fix = (Boolean)a.fix;
            Model = a;
        }
        public int Fontsize
        {
            set
            {
                fontsize = value;
            }
            get
            {
                return fontsize;
            }
        }
        public AssignmentState State
        {
            set
            {
                if (state != value)
                {
                    state = value;
                    RaisePropertyChanged("State");
                    RaisePropertyChanged("StateText");
                }
                if (state == AssignmentState.proposal)
                {
                    Opacity = 0.5;
                }
                else
                {
                    Opacity = 1;
                }
            }
            get
            {
                return state;
            }

        }
        /// <summary>
        /// BorderBrush is needed in View to change the border of the assignment
        /// </summary>
        public Brush BorderBrush
        {
            get
            {
                if (IsSelected)
                {
                    return Brushes.Red;
                }
                else
                {
                    SolidColorBrush sb = new SolidColorBrush();
                    
                    if (Type == (int)AssignmentType.help)
                    {
                        Color c = new Color();
                        c.R = 0;
                        c.G = 0;
                        c.B = 0;
                        c.A = 255;
                        sb.Color = c;
                        sb.Opacity = 1;
                        
                        return sb;
                    }
                    else
                    {
                        if (Type == (int)AssignmentType.bar)
                        {
                            Color c = new Color();
                            c.R = 255;
                            c.G = 255;
                            c.B = 255;
                            c.A = 255;
                            sb.Color = c;
                            sb.Opacity = 1;
                            return sb;
                        }
                        else
                        {
                            if (Type == (int)AssignmentType.failure)
                            {
                                Color c = new Color();
                                c.R = 237;
                                c.G = 242;
                                c.B = 152;
                                c.A = 100;
                                sb.Color = c;
                                sb.Opacity = 1;
                                return sb;
                            }
                            else
                            {
                                Color c = new Color();
                                c.R = 152;
                                c.G = 242;
                                c.B = 160;
                                c.A = 100;
                                sb.Color = c;
                                return sb;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// delivers the color (as brush) for the view of the assignment
        /// </summary>
        public Brush BackgroundBrush
        {
            get
            {
                SolidColorBrush sb = new SolidColorBrush();
                if (Type == (int)AssignmentType.bar)
                {
                    Color c = new Color();
                    c.A = 255;
                    c.B = 245;
                    c.G = 245;
                    c.R = 245;
                    sb.Color = c;

                    return sb;
                }
                else
                {
                    if (Type == (int)AssignmentType.failure)
                    {

                        Color c = new Color();
                        c.R = 235;
                        c.G = 229;
                        c.B = 255;
                        c.A = 255;
                        sb.Color = c;
                        sb.Opacity = 1;
                        return sb;
                    }
                    else
                    {
                        Color c = new Color();
                        c.R = 229;
                        c.G = 249;
                        c.B = 255;
                        c.A = 255;
                        sb.Color = c;
                        return sb;
                    }
                }
            }
        }

        public Boolean IsDragging 
        {
            set
            {
                if (isDragging != value)
                {
                    isDragging = value;


                    if (isDragging)
                        Opacity = 0.7;
                    else
                        Opacity = 1;
                    RaisePropertyChanged("IsDragging");
                    RaisePropertyChanged("ZIndex");
                }
            }
            get
            {
                return isDragging;
            }
        }
        public Boolean IsSelected
        {
            set
            {
                if (isSelected != value)
                {
                    isSelected = value;
                    RaisePropertyChanged("IsSelected");
                    RaisePropertyChanged("BorderBrush");
                }
            }
            get
            {
                return isSelected;
            }
        }
        
        /// <summary>
        /// true, when the assignment is fixed to a date
        /// </summary>
        public Boolean Fix
        {
            set
            {
                if (Fix != value)
                {
                    Model.fix = value;
                    RaisePropertyChanged("Fix");
                    RaisePropertyChanged("CornerRadius");
                }
            }
            get
            {
                return (Boolean)Model.fix;
            }
        }

        /// <summary>
        /// Property to display text in the timeline
        /// </summary>
        public string Timelinetext  {  get;  set;  }
        /// <summary>
        /// returns the type of the assignmen (1 maintenance, 2 failure)
        /// </summary>
        public int Type   { get; set;    }

        /// <summary>
        /// ZIndex of the assignment, to bring it into front while dragging
        /// </summary>
        public int ZIndex 
        {
            get
            {
                if (isDragging)
                    return 99;
                else
                    return 2;
            }

        }
        
        public EmployeeViewmodel Employee
        {
            set
            {
                if (employee != value)
                {
                    employee = value;
                    RaisePropertyChanged("Employee");
                    Y = employee.Y;
                    Height = employee.Height;
                }
            }
            get
            {
                return employee;
            }
        }
        private double dayWidth()
        {
            TimeSpan daylength = src.EndTimePerDay.Subtract(src.StartTimePerDay);
            double dayWidth = (daylength.Hours * 60 + daylength.Minutes) * src.PixelPerMinute + src.DistancePixel;
            return dayWidth;    
        }
        /// <summary>
        /// sets the x coordinate of the assignment, the FromDate is also set
        /// </summary>
        public double X
        {
            set
            {
                if (x != value)
                {
                    x = value;
                    RaisePropertyChanged("X");

                    double dayWidth = this.dayWidth();
                    double factor = (x / dayWidth);
                    double days, weekends, rest;
                    DateTime help;

                    int dayFromDate = (int)Model.FromDate.DayOfWeek;
                    int dayStartDate = (int)src.StartDate.DayOfWeek;

                    // the startDate is smaller than the fromDate
                    if (factor > 0)
                    {
                        days = Math.Floor(factor);
                        rest = x - days * dayWidth;

                        weekends = Math.Floor(days / 5);
                        //Saturday and Sunday are not shown in timeline -> add it to the date                
                        //correct the date, when an weekend is between                
                        if (dayStartDate + (days - weekends * 5) > 5)
                            weekends++;
                        days = days + weekends * 2;

                    }
                    else
                    {
                        // in this branch, everything is calculated into the opposite direction
                        days = Math.Round(factor + 0.5);
                        rest = x - days * dayWidth;

                        weekends = Math.Floor(Math.Abs(days / 5));
                        if (dayStartDate + (days + weekends * 5) <= 1)
                            weekends++;
                        days = days - weekends * 2 - 1;
                        rest = dayWidth + rest;
                    }

                    help = src.StartDate.AddDays(days);
                    help = help.AddMinutes(rest / src.PixelPerMinute);
                    Model.FromDate = help;

                    RaisePropertyChanged("From");
                    Model.ToDate = Model.FromDate.AddMinutes(duration);
                    RaisePropertyChanged("ShortTimeString");
                }
            }
            get  {  return x;  }
        }
        
        /// <summary>
        /// Set the Y coordinate of the assignment
        /// </summary>
        public double Y
        {
            set
            {
                if (y != value)
                {
                    y = value;
                    RaisePropertyChanged("Y");
                }
            }
            get  {   return y;   }
        }
        
        /// <summary>
        /// set the FromDate of the assignment, the X coordinate is also set
        /// </summary>
        public DateTime From
        {
            set
            {
                if (Model.FromDate != value)
                {
                    double helper = 0;
                    Model.FromDate = value;
                    RaisePropertyChanged("From");
                    double dayWidth = this.dayWidth();
                    double weekends = 0;
                    int startweekday = (int)src.StartDate.DayOfWeek;
                    int fromdateweekday = (int)Model.FromDate.DayOfWeek;
                    TimeSpan diff = Model.FromDate - src.StartDate;

                    if (diff.Hours < 0)
                    {

                        //add the time between start and end (the night)                    
                        TimeSpan night = src.EndTimePerDay - src.StartTimePerDay;
                        diff = diff.Add(night);
                        helper = src.DistancePixel;
                        //when hours are negative, only a part of a day has to be added
                        //but when the weekens has to be take in account, we have to take care 
                        //about this - so we add one day to the fromdateweekday -> later the number 
                        //of weekends will be correct
                        fromdateweekday++;

                    }
                    double days = diff.Days;
                    //Saturday and Sunday are not shown in the Timeline -> subtract them from days
                    if (Model.FromDate < src.StartDate)
                    {
                        weekends = Math.Floor(Math.Abs(days / 7f));

                        if (fromdateweekday > startweekday)
                            weekends++;

                        days = days + weekends * 2;
                    }
                    else
                    {
                        weekends = Math.Floor(days / 7);
                        if (fromdateweekday < startweekday)
                            weekends++;
                        days = days - weekends * 2;

                    }

                    x = (diff.Hours * 60 + diff.Minutes) * src.PixelPerMinute + days * dayWidth - helper;
                    RaisePropertyChanged("X");
                    To = Model.FromDate.AddMinutes(duration);
                    RaisePropertyChanged("ShortTimeString");
                    RaisePropertyChanged("FromDateText");
                }
            }
            get
            {
                return Model.FromDate;
            }
        }
        
        /// <summary>
        /// set the to date
        /// </summary>
        public DateTime To
        {
            set
            {
                Model.ToDate = value;
                RaisePropertyChanged("To");
                TimeSpan diff = Model.ToDate - Model.FromDate;
                Duration = diff.Minutes + diff.Hours * 60 + diff.Days * 3600;              
            }
            get  {    return Model.ToDate;      }
        }
        /// <summary>
        /// set the width of the assignment, the duration is also set
        /// </summary>
        public double Width
        {
            set
            {
                width = value;
                RaisePropertyChanged("Width");
                duration = width / src.PixelPerMinute;
                RaisePropertyChanged("Duration");
            }
            get { return width; }

        }
        /// <summary>
        /// set the duration, the width is also set
        /// </summary>
        public double Duration
        {
            set
            {
                duration = value;
                RaisePropertyChanged("Duration");
                width = duration * src.PixelPerMinute;
                RaisePropertyChanged("Width");
                //To = From.AddMinutes(duration);
            }
            get { return duration; }

        }
        /// <summary>
        /// set the height of the assignment
        /// </summary>
        public double Height
        {
            set
            {
                height = value;
                RaisePropertyChanged("Height");
            }
            get
            {
                return height;
            }
        }        
        /// <summary>
        /// set the assignment to the next employee using the y coordinate
        /// </summary>
        public void SetToNextEmployee()
        {
            Employee = src.GetEmployeebyY(Y);
            Y = Employee.Y;
        }

        /// <summary>
        /// round the time of the assignment
        /// </summary>
        public void RoundMinutes()
        {
            int i = (int) Math.Round( (double)From.Minute / src.RoundToMinutes);
            int minute = i * src.RoundToMinutes;
            if(minute != 60)
                From = new DateTime(From.Year, From.Month, From.Day, From.Hour, minute, 0);
            else
                From = new DateTime(From.Year, From.Month, From.Day, From.Hour + 1, 0, 0);
            
        }
        /// <summary>
        /// delivers an assignemnt object
        /// </summary>
        /// <returns>assignment of type Assignment (Model)</returns>
        public Assignment GetAssignmentObject()
        {
            Assignment a = new Assignment();
            a.Employee = this.Employee.User;
            a.fix = this.Model.fix;
            a.FromDate = this.From;
            a.IBase = this.Model.Ibase;
            //a.ibase = this.ibase;
            a.Id = this.Model.Id;
            a.ToDate = this.Model.ToDate;
            a.Type = (short)this.Type;
            a.State = (int)this.State;
            return a;
        }

        /// <summary>
        /// set the opacity of the assignment
        /// </summary>
        public double Opacity
        {
            set
            {
                _opacity = value;
                RaisePropertyChanged("Opacity");
            }
            get
            {
                return _opacity;
            }
        }

        #region read-only attributes
        public String IBaseAdress
        {
            get
            {
                return Model.Street + " " + Model.HNr;
            }
        }
        
        public String ShortTimeString
        {
            get
            {
                return Model.FromDate.ToShortTimeString() + " - " + Model.ToDate.ToShortTimeString();
            }
            set
            {
                string fromstr, tostr, val = value;
                if (value.Contains("-"))
                {
                    fromstr = value.Substring(0, value.IndexOf("-") - 1);
                    tostr = value.Substring(value.IndexOf("-") + 1 );
                    TimeSpan f = TimeSpan.Parse(fromstr);
                    TimeSpan t = TimeSpan.Parse(tostr);
                    To = new DateTime(To.Year, To.Month, To.Day, t.Hours, t.Minutes, 0);
                    DateTime from = new DateTime(From.Year,From.Month,From.Day,f.Hours,f.Minutes,0);
                    RaisePropertyChanged("ShortTimeString");
                    
                    From = from;
                    
                }
            }
        }
        public String FromDateText
        {
            get
            {
                return From.Date.ToShortDateString();
            }
            set
            {
                DateTime help = DateTime.Parse(value);
                To = new DateTime(help.Year, help.Month, help.Day, To.Hour, To.Minute, To.Second);
                From = new DateTime(help.Year, help.Month, help.Day, From.Hour, From.Minute, From.Second);
                RaisePropertyChanged("FromDateText");
            }
        }
        public String StateText
        {
            get
            {
                switch (state)
                {
                    case AssignmentState.finished: return "Finished";
                    case AssignmentState.inform: return "Informed";
                    case AssignmentState.open: return "Open";
                    
                }
                return "";
            }
            set
            {
                if (value == "Finished")
                    State = AssignmentState.finished;
                if (value == "Informed")
                    State = AssignmentState.inform;
                if (value == "Open")
                    State = AssignmentState.open;
            }
        }
        public String TypeText
        {
            get
            {
                switch (Type)
                {
                    case (int) AssignmentType.failure: return "Failure";
                    case (int) AssignmentType.maintenance: return "Maintenance";
                }
                return "";
            }
        }
        public int CornerRadius
        {
            get
            {
                if ((bool)Model.fix)
                    return 0;
                else
                    return 10;
            }
        }
        #endregion


    }
}
