﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using APlan.Service;
using System.Windows.Media;
using GalaSoft.MvvmLight;

namespace APlan.ViewModel
{
    /// <summary>
    /// represents an employee (Viewmodel)
    /// An Employee can be an technician and a user of A-Plan
    /// </summary>
    public class EmployeeViewmodel : ViewModelBase 
    {        
        private double x, y, width, height;
        int posnr;
        private MainViewModel src;
        public EmployeeViewmodel(MainViewModel srcinp)
        {
            src = srcinp;
            Height = src.ElementHeight + src.ElementDistance;
            x = -930;
            width = 2000;            
        }
        /// <summary>
        /// Windows User name
        /// </summary>
        public String User { get; set; }
        /// <summary>
        /// Name of the user
        /// </summary>
        public String Name { get; set; }
        /// <summary>
        /// City, where the employee lives
        /// </summary>
        public City City { get; set; }
        
        /// <summary>
        /// true, if the employee is technician
        /// </summary>
        public Boolean Technician { get; set; }

        /// <summary>
        /// return the name of the city
        /// </summary>
        public string CityDescr
        {
            get
            {
                if (City != null)
                    return City.City1;
                else
                    return "";
            }
        }

        /// <summary>
        /// returns a brush for the visualization of the Employee in the view
        /// </summary>
        public Brush Backgroundbrush
        {
            get
            {
                SolidColorBrush sb = new SolidColorBrush();
                if ((posnr % 2) == 0)
                {
                    Color c = new Color();
                    c.A = 0;
                    c.B = 255;
                    c.G = 255;
                    c.R = 255;
                    sb.Color = c;

                    return sb;

                }
                else
                {
                    Color c = new Color();
                    c.A = 255;
                    c.B = 245;
                    c.G = 245;
                    c.R = 245;
                    sb.Color = c;

                    return sb;
                }
            }
        }

        /// <summary>
        /// return the position of the employee
        /// </summary>
        public int PosNr
        {
            set
            {
                posnr = value;
                RaisePropertyChanged("PosNr");
                Y = (posnr - 1) * (src.ElementHeight + src.ElementDistance) + src.DistanceToFirstElement;
            }
            get
            {
                return posnr;
            }

        }

        /// <summary>
        /// get/set the height of the employee
        /// </summary>
        public double Height
        {
            set
            {
                height = value;
                RaisePropertyChanged("Height");
            }
            get
            {
                return height;
            }
        }

        /// <summary>
        /// get/set the width of the employee
        /// </summary>
        public double Width
        {
            private set
            {
                width = value;
                RaisePropertyChanged("Width");
            }
            get
            {
                return width;
            }
        }

        /// <summary>
        /// get/set the x coordinate
        /// </summary>
        public double X
        {
            set
            {
                x = value;
                RaisePropertyChanged("X");
            }
            get
            {
                return x;
            }
        }

        /// <summary>
        /// get/set the y coordinate
        /// </summary>
        public double Y
        {
            set
            {
                y = value;
                RaisePropertyChanged("Y");
            }

            get
            {
                return y;
            }
        }

    }
}
