﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System.Collections.ObjectModel;
using APlan.Service;
using System.Threading;
using System.ServiceModel;
using System;
using System.Windows.Data;
using System.Data.Linq;
using System.Linq;
using System.Collections.Generic;
using System.Configuration;
using System.Security.Principal;
using System.Windows.Input;

namespace APlan.ViewModel
{
    /// <summary>
    /// This class contains properties that the main View can data bind to.        
    /// You can also use Blend to data bind with the tool's support.        
    /// </summary>
    
    public class MainViewModel : ViewModelBase, IAPlanCallback
    {        
        /// <summary>
        /// The <see cref="Customers" /> property's name.
        /// </summary>
        public const string CustomersPropertyName = "Customers";
        private ObservableCollection<CustIBaseViewModel> _cust;
        private ObservableCollection<CustIBaseViewModel> _openFailures;
        private ObservableCollection<UserViewModel> _userlist;
        private ObservableCollection<HeatMapViewModel> _openCustHeatMap;
        private ObservableCollection<MessageViewModel> _messages;
        private CollectionView _custCollView;
        private CollectionView _openFailureCollView;
        private IAPlanService proxy;
        private SynchronizationContext sync;
        // the Employees and Assignments are saved in a dictionary too for faster access
        public Dictionary<string, EmployeeViewmodel> Employeedict;
        private Dictionary<long, AssignmentViewModel> Assignmentdict;
        private long _timeRangeFrom, _timeRangeTo;
        private DateTime _assignmentSettingDateFrom, _assignmentSettingDateTo;
        private Boolean _isBusy, _routePlanProposed;
        private AssignmentViewModel _actualSelectedAssignment;
        private int _roundToMinutes = 15;
        private int _defaultAssignmentDuration = 90;
        private Double _canvasScaleFactor = 1;
        
        private DateTime? _selectedDate =null;
        private Boolean _undohelp = false;
        private Boolean _undopossible, _redopossible;
        private Boolean _distributedAssignment = false;
        #region Properties

        public DateTime RouteplanStartDate
        {
            get;
            set;
        }

        public DateTime RouteplanEndDate
        {
            get;
            set;
        }
        /// <summary>
        /// The states of assignments for the combo box on the main window
        /// </summary>
        public String[] AssignmentStates
        {
            get
            {
                string[] states = new string[3];
                states[0] = "Finished";
                states[1] = "Informed";
                states[2] = "Open";
                return states;
            }
        }

        /// <summary>
        /// Zoom factor for the canvas
        /// </summary>
        public Double CanvasScaleFactor
        {
            get
            {
                return _canvasScaleFactor;
            }
            set
            {
                if (_canvasScaleFactor != value)
                {
                    _canvasScaleFactor = value;
                    RaisePropertyChanged("CanvasScaleFactor");
                }
            }
        }
        /// <summary>
        /// Selected date for paning 
        /// </summary>
        public DateTime? SelectedDate
        {
            get
            {
                return _selectedDate;
            }
            set
            {
                if (_selectedDate != value)
                {
                    _selectedDate = value;
                    if (value != null)
                    {
                        DateTime help = (DateTime)value;
                        RaisePropertyChanged("SelectedDate");
                        AssignmentViewModel av = new AssignmentViewModel(this);
                        if (value < DateTime.Now)
                            av.From = help.AddDays(1);
                        else
                            av.From = (DateTime)value;
                        GalaSoft.MvvmLight.Messaging.Messenger.Default.Send<AssignmentViewModel>(av);

                        _selectedDate = null;
                    }
                }
            }
        }

        public int DefaultAssignmentDuration
        {
            get
            {
                return _defaultAssignmentDuration;
            }
            set
            {
                if (_defaultAssignmentDuration != value)
                {
                    _defaultAssignmentDuration = value;
                    RaisePropertyChanged("DefaultAssignmentDuration");
                }
            }
        }
        /// <summary>
        /// set the value for rounding date assignment dates
        /// for example: 15 round to 00, 15, 30 and 45 minutes
        /// </summary>
        public int RoundToMinutes
        {
            get
            {
                return _roundToMinutes;
            }
            set
            {
                if (_roundToMinutes != value)
                {
                    _roundToMinutes = value;
                    RaisePropertyChanged("RoundToMinutes");
                }
            }
        }
        /// <summary>
        /// number of open failures available
        /// </summary>
        public int NumberOpenFailures
        {
            get
            {
                return OpenFailureCollectionView.Count;
            }
        }

        public ObservableCollection<MessageViewModel> Messages
        {
            get
            {
                return _messages;
            }
            set
            {
                _messages = value;
                RaisePropertyChanged("Messages");
            }
        }

        public ObservableCollection<HeatMapViewModel> OpenCustHeatMap
        {
            get
            {
                return _openCustHeatMap;
            }
            set
            {
                _openCustHeatMap = value;
                RaisePropertyChanged("OpenCustHeatMap");
            }
        }

        public ObservableCollection<UserViewModel> UserList
        {
            get
            {
                return _userlist;
            }
            set
            {
                if (_userlist != value)
                {
                    _userlist = value;
                    RaisePropertyChanged("UserList");
                }
            }
        }

        public ObservableCollection<CustIBaseViewModel> OpenFailures
        {
            get
            {
                return _openFailures;
            }

            set
            {
                if(_openFailures != value)
                {
                    _openFailures = value;
                    RaisePropertyChanged("OpenFailures");
                    RaisePropertyChanged("NumberOpenFailures");
                }
            }
        }

        /// <summary>
        /// Gets the Customers property.
        /// 
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public ObservableCollection<CustIBaseViewModel> Customers
        {
            get
            {
                return _cust;
            }

            set
            {
                if (_cust == value)
                {
                    return;
                }

                var oldValue = _cust;
                _cust = value;

                RaisePropertyChanged(CustomersPropertyName);
            }
        }

        public CollectionView OpenFailureCollectionView
        {
            get
            {
                return _openFailureCollView;
            }
            set
            {
                _openFailureCollView = value;
                RaisePropertyChanged("OpenFailureCollectionView");
            }
        }
        public CollectionView CustomerCollectionView 
        {
            get
            {
                return _custCollView;
            }
            set
            {
                _custCollView = value;
                RaisePropertyChanged("CustomerCollectionView");
            }
        }

        public Boolean IsBusy
        {
            get
            {
                return _isBusy;
            }
            set
            {
                _isBusy = value;
                RaisePropertyChanged("IsBusy");
            }
        }

        public Boolean RoutePlanProposed
        {
            get
            {
                return _routePlanProposed;
            }
            set
            {
                _routePlanProposed = value;
                RaisePropertyChanged("RoutePlanProposed");
            }
        }

        public System.DateTime StartDate { get; set; }
        public TimeSpan StartTimePerDay { get; set; }
        public TimeSpan EndTimePerDay { get; set; }
        public double DistancePixel { get; set; }
        public double PixelPerMinute { get; set; }
        public double ElementHeight { get; set; }
        public double ElementDistance { get; set; }
        public double DistanceToFirstElement { get; set; }

        #region Attributes for Assignment settings
        public DateTime AssignmentSettingDateFrom
        {
            set
            {
                _assignmentSettingDateFrom = value;
                RaisePropertyChanged("AssignmentSettingDateFrom");
            }
            get
            {
                return _assignmentSettingDateFrom;
            }
        }
        public DateTime AssignmentSettingDateTo
        {
            set
            {
                _assignmentSettingDateTo = value;
                RaisePropertyChanged("AssignmentSettingDateTo");
            }
            get
            {
                return _assignmentSettingDateTo;
            }
        }
        public Int64 TimeRangeFrom
        {
            set
            {
                _timeRangeFrom = value;
                RaisePropertyChanged("TimeRangeFrom");
                RaisePropertyChanged("TimeRangeString");
            }
            get
            {
                return _timeRangeFrom;
            }
        }
        public Int64 TimeRangeTo
        {
            set
            {
                _timeRangeTo = value;
                RaisePropertyChanged("TimeRangeTo");
                RaisePropertyChanged("TimeRangeString");
            }
            get
            {
                return _timeRangeTo;
            }
        }
        public String TimeRangeString
        {
            get
            {
                TimeSpan fr = new TimeSpan(_timeRangeFrom);
                TimeSpan to = new TimeSpan(_timeRangeTo);
                int round = (fr.Minutes / 15) * 15;
                fr = new TimeSpan(fr.Hours, (fr.Minutes / 15) * 15,0);
                
                string fromMinutes = fr.Minutes.ToString("00");
                return fr.Hours.ToString() +  ":" + fromMinutes + " - " + to.Hours + ":" + to.Minutes.ToString("00");
            }
        }
        #endregion

        public ObservableCollection<AssignmentViewModel> Assignments { set; get; }
        public ObservableCollection<AssignmentViewModel> TimelineData { set; get; }
        public ObservableCollection<EmployeeViewmodel> Employees { set; get; }
        #endregion

        
        public RelayCommand<string> SearchCustomers
        {
            get;
            private set;
        }

        public RelayCommand<CustIBaseViewModel> UpdateIBaseCommand
        {
            get;
            private set;
        }

        public RelayCommand<CustIBaseViewModel> InsertAssignmentCommand
        {
            get;
            private set;
        }

        public RelayCommand<Object> RemoveAssignmentCommand
        {
            get;
            private set;
        }

        public RelayCommand<Object> GenerateTourPlanCommand
        {
            get;
            private set;
        }

        public RelayCommand<Object> AcknowledgeTourPlan
        {
            get;
            private set;
        }
        
        public RelayCommand<CustIBaseViewModel> InsertOpenFailureCommand
        {
            get;
            private set;
        }
        
        public RelayCommand<Object> UndoCommand
        {
            get;
            private set;
        }
        
        public RelayCommand<Object> RedoCommand
        {
            get;
            private set;
        }

        #region execute methods for commands

        private void executeInsertAssignmentCommand(CustIBaseViewModel c)
        {
            TimeSpan tfrom = new TimeSpan(_timeRangeFrom);
            TimeSpan tto = new TimeSpan(_timeRangeTo);
            proxy.CreateAssignment(c.GetCustIBaseView(), _assignmentSettingDateFrom , _assignmentSettingDateTo,
                tfrom, tto);
        }

        private void executeInsertOpenFailureCommand(CustIBaseViewModel c)
        {
            proxy.CreateOpenFailure(c.Model.Ibase);
        }

        private void executeRemoveAssignmentCommand(Object obj)
        {
            if (obj.GetType().ToString() == "APlan.ViewModel.AssignmentViewModel")
            {
                AssignmentViewModel av = (AssignmentViewModel)obj;
                av = Assignmentdict[av.Model.Id];
                Assignmentdict.Remove(av.Model.Id);
                //Assignments.Remove(av);
                CustIBaseViewModel cv = (CustIBaseViewModel)CustomerCollectionView.CurrentItem;
                if (cv != null)
                {
                    var avhelp = (from a in cv.AssignmentViewmodels
                                  where a.Model.Id == av.Model.Id
                                  select a).Single();

                    cv.AssignmentViewmodels.Remove(avhelp);
                }
                if (!_undohelp)
                    proxy.RemoveAssignment(av.GetAssignmentObject());

                av.Width = 0;
                av.Height = 0;
                //av.From = new DateTime(1970, 1, 1);
                av.Model.Id = 0;
            }
        }

        private void executeGenerateTourPlanCommand(Object selectedItems)
        {
            System.Collections.IList items = (System.Collections.IList)selectedItems;
            var coll = items.Cast<HeatMapViewModel>();
            HeatMapViewModel[] hv = (HeatMapViewModel[])coll.ToArray();
            proxy.CreateRoutePlan(hv, RouteplanStartDate, RouteplanEndDate, StartTimePerDay, EndTimePerDay);
            IsBusy = true;
        }
        private void executeAcknowledgeTourPlan(Object ack)
        {
            String param = (String)ack;
            if (param == "True")
                proxy.AcknowledgeRoutePlan(true);
            else
                proxy.AcknowledgeRoutePlan(false);
            RoutePlanProposed = false;
        }
        private void executeUpdateCommand(CustIBaseViewModel p)
        {
            if(p!=null)
                proxy.UpdateIBase(p.GetCustIBaseView());
        }

        #endregion
        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel()
        {
            
            if (IsInDesignMode)
            {
                
            }
            else
            {                
                StartDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, 7, 0, 0);
                int weekday = (int) StartDate.DayOfWeek;
                if (weekday > 5)
                {
                    weekday = 8 - weekday;
                    StartDate =  StartDate.AddDays(weekday);
                }
                AssignmentSettingDateFrom = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, 7, 0, 0);
                AssignmentSettingDateTo = AssignmentSettingDateFrom.AddDays(14);
                StartTimePerDay = new TimeSpan(7, 0, 0);
                EndTimePerDay = new TimeSpan(19, 0, 0);
                TimeRangeFrom = StartTimePerDay.Ticks;
                TimeRangeTo = EndTimePerDay.Ticks;
                PixelPerMinute = 1.1;
                DistancePixel = 50;
                ElementHeight = 50;
                ElementDistance = 10;
                DistanceToFirstElement = 15;
                _routePlanProposed = false;
                UserList = new ObservableCollection<UserViewModel>();
                Employees = new ObservableCollection<EmployeeViewmodel>();
                Assignments = new ObservableCollection<AssignmentViewModel>();
                TimelineData = new ObservableCollection<AssignmentViewModel>();
                Employeedict = new Dictionary<string, EmployeeViewmodel>();
                Assignmentdict = new Dictionary<long, AssignmentViewModel>();
                
                OpenFailures = new ObservableCollection<CustIBaseViewModel>();
                OpenFailureCollectionView = new ListCollectionView(OpenFailures);
                Customers = new ObservableCollection<CustIBaseViewModel>();
                CustomerCollectionView = new ListCollectionView(Customers);
                CustomerCollectionView.CurrentChanged += new EventHandler(CustomerCollectionView_CurrentChanged);
                sync = SynchronizationContext.Current;
                InstanceContext instcont = new InstanceContext(this);
                string url = ConfigurationManager.AppSettings["ServerUrl"];
                NetTcpBinding tcpbind = new NetTcpBinding();
                tcpbind.CloseTimeout = new TimeSpan(0, 20, 0);
                tcpbind.MaxReceivedMessageSize = 2147483640;
                tcpbind.ReceiveTimeout = new TimeSpan(0, 20, 0);
                tcpbind.MaxBufferSize = 1235465536;
                
                var factory = new DuplexChannelFactory<IAPlanService>(instcont, tcpbind, new EndpointAddress(url));
               //factory.Credentials.Windows.ClientCredential = new System.Net.NetworkCredential("user", "pass");
                
                proxy = factory.CreateChannel();
                
                GalaSoft.MvvmLight.Messaging.Messenger.Default.Register<AssignmentViewModel>(this, OnSelectedAssignmentChanged);
                
                _undopossible = true;
                _redopossible = true;
                SearchCustomers = new RelayCommand<string>(
                    m => { proxy.GetCustomer(m);
                           IsBusy = true; },
                    m => !IsBusy);

                UpdateIBaseCommand = new RelayCommand<CustIBaseViewModel>(
                m => executeUpdateCommand(m),
                m => true);

                InsertAssignmentCommand = new RelayCommand<CustIBaseViewModel>(
                    m => executeInsertAssignmentCommand(m),
                    m => CustomerCollectionView.CurrentItem != null);

                InsertOpenFailureCommand = new RelayCommand<CustIBaseViewModel>(
                    m => executeInsertOpenFailureCommand(m),
                    m => CustomerCollectionView.CurrentItem != null);
                
                RemoveAssignmentCommand = new RelayCommand<Object>(
                    m => executeRemoveAssignmentCommand(m),
                    m => true);
                
                GenerateTourPlanCommand = new RelayCommand<Object>(
                    m => executeGenerateTourPlanCommand(m),
                    m => RoutePlanProposed == false);
                AcknowledgeTourPlan = new RelayCommand<Object>(
                    m => executeAcknowledgeTourPlan(m),
                    m => RoutePlanProposed == true );
                
                UndoCommand = new RelayCommand<Object>(
                    m => proxy.UndoRedo(true),
                    m =>_undopossible);
                
                RedoCommand = new RelayCommand<Object>(
                    m => proxy.UndoRedo(false),
                    m => _redopossible);
                 
            }
            
        }
        /// <summary>
        /// OnSelectedAssignmentChanged is called, when a message is send via Messagnerwith 
        /// an AssignmentViewModel as parameter
        /// </summary>
        /// <param name="a">AssignmentViewModel</param>
        private void OnSelectedAssignmentChanged(AssignmentViewModel a)
        {         
            if (_actualSelectedAssignment != null)
                _actualSelectedAssignment.IsSelected = false;
            _actualSelectedAssignment = a;
            a.IsSelected = true;
        }

        /// <summary>
        /// the method is called, when the user change the current item of the CustomerCollectionView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void CustomerCollectionView_CurrentChanged(object sender, EventArgs e)
        {
            
            CollectionView c = (CollectionView)sender;
            if (c.CurrentItem != null)
            {
                CustIBaseViewModel vm = (CustIBaseViewModel)c.CurrentItem;
                selectAssignmentFromCurrentCustomer(vm.Model);
            }
            
        }

        /// <summary>
        /// select the assignment with the highest date from the given customer
        /// </summary>
        /// <param name="vm">customer</param>
        private void selectAssignmentFromCurrentCustomer(CustIBaseView vm)
        {
            DateTime highestDate = DateTime.MinValue;
            Assignment assign = null;
            foreach (Assignment a in vm.Assignments)
            {
                if (a.FromDate.Year != 9999)
                {
                    if (a.FromDate > highestDate)
                    {
                        highestDate = a.FromDate;
                        assign = a;
                    }
                }
            }
            selectAssignment(assign);
        }

        /// <summary>
        /// send a message to the assignemntview to show the given assignment
        /// </summary>
        /// <param name="assign"></param>
        private void selectAssignment(Assignment assign)
        {
            if (assign != null)
            {
                AssignmentViewModel av = ViewModelLocator.MainStatic.Assignmentdict[assign.Id];
                if (_actualSelectedAssignment != av)
                {
                    if (_actualSelectedAssignment != null)
                    {
                        _actualSelectedAssignment.IsSelected = false;
                    }
                    _actualSelectedAssignment = av;
                    if (!av.IsSelected)
                    {
                        av.IsSelected = true;
                        GalaSoft.MvvmLight.Messaging.Messenger.Default.Send<AssignmentViewModel>(av);
                    }
                }
            }
        }

        /// <summary>
        /// get the nearest employee to given y
        /// </summary>
        /// <param name="y"></param>
        /// <returns></returns>
        public EmployeeViewmodel GetEmployeebyY(double y)
        {
            int pos = 1;
            EmployeeViewmodel emp = null;
            double dist = ElementDistance + ElementHeight;
            y += ElementHeight;
            pos = (int)Math.Round(y / dist);
            if (pos > Employees.Count)
                pos = Employees.Count;
            if (pos < 1)
                pos = 1;
            foreach (EmployeeViewmodel e in Employees)
            {
                if (e.PosNr == pos)
                    return e;
            }

            return emp;
        }

        /// <summary>
        /// do initial data loading
        /// </summary>
        public void LoadData()
        {
            try
            {
                OpenCustHeatMap = new ObservableCollection<HeatMapViewModel>(proxy.GetOpenCustomers());
                proxy.GetEmployees();
                proxy.GetAssignments(new DateTime(2009, 1, 1), new DateTime(2012, 1, 1));
                proxy.Subscribe(WindowsIdentity.GetCurrent().Name);
                proxy.GetOpenFailures();
            }
            catch (Exception e)
            {

            }
        }

        /// <summary>
        /// deliver detail (customer) information to heat map
        /// </summary>
        /// <param name="h"></param>
        /// <returns></returns>
        public HeatMapViewModel UpdateOpenCustHeatMap(HeatMapViewModel h)
        {
            return proxy.GetOpenCustomersDetail(h);
        }

        /// <summary>
        /// insert given assignment
        /// </summary>
        /// <param name="a"></param>
        public void InsertDragDropAssignment(Assignment a)
        {
            proxy.InsertAssignment(a);
        }

        /// <summary>
        /// remove open failure (given by key) 
        /// </summary>
        /// <param name="ibase">ibase id</param>
        public void RemoveOpenFailure(Int64 ibase)
        {
            proxy.RemoveOpenFailure(ibase);
        }

        /// <summary>
        /// unsubscribe user on close of the application
        /// </summary>
        public void Close()
        {
            proxy.Unsubscribe(WindowsIdentity.GetCurrent().Name);
        }

        /// <summary>
        /// generate background lines (time ticks) and text
        /// </summary>
        private void generateBackgroundLines()
        {
            DateTime d = StartDate;
            foreach (EmployeeViewmodel ev in Employees)
            {
                if (ev.PosNr % 2 != 0)
                {
                    DateTime from = new DateTime(2009, 1, 1, 0, 0, 0);
                    DateTime to = new DateTime(2012, 12, 31, 0, 0, 0);
                    AssignmentViewModel a = new AssignmentViewModel(this);
                    a.From = from;
                    a.To = to;
                    a.Height = ElementHeight;
                    a.Y = ev.Y;
                    a.Type = (int)AssignmentType.bar;
                    Assignments.Add(a);
                }
            }
            for (int j = 1; j <= 2; j++)
            {
                d = StartDate;
                if (j == 2)
                    d = StartDate.AddDays(-1);
                for (int i = 1; i < 50; i++)
                {
                    if (d.DayOfWeek != DayOfWeek.Saturday && d.DayOfWeek != DayOfWeek.Sunday)
                    {
                        DateTime from = new DateTime(d.Year, d.Month, d.Day, StartTimePerDay.Hours, 0, 0);
                        DateTime end = new DateTime(d.Year, d.Month, d.Day, EndTimePerDay.Hours, 0, 0);
                        DateTime mid = new DateTime(d.Year, d.Month, d.Day, 12, 0, 0);
                        DateTime late = new DateTime(d.Year, d.Month, d.Day, 16, 0, 0);
                        AssignmentViewModel a = new AssignmentViewModel(this);
                        a.From = from;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 0;                        
                        a.Width = 1;
                        a.Height = 3000;
                        Assignments.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = end;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 0;                        
                        a.Width = 1;
                        a.Height = 3000;
                        Assignments.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = mid;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 0;                        
                        a.Width = 1;
                        a.Height = 3000;
                        Assignments.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = late;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 0;                        
                        a.Width = 1;
                        a.Height = 3000;
                        Assignments.Add(a);

                        // Timeline ticks
                        a = new AssignmentViewModel(this);
                        a.From = from;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 35;                        
                        a.Width = 1;
                        a.Height = 50;
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = mid;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 35;                        
                        a.Width = 1;
                        a.Height = 50;
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = late;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 35;                       
                        a.Width = 1;
                        a.Height = 50;
                        TimelineData.Add(a);

                        // Timeline ticks
                        a = new AssignmentViewModel(this);
                        a.From = end;
                        a.Type = (short)AssignmentType.help;
                        a.Y = 35;                        
                        a.Width = 1;
                        a.Height = 3000;
                        TimelineData.Add(a);

                        //Timeline text
                        a = new AssignmentViewModel(this);
                        a.From = from;
                        a.Type = (short)AssignmentType.text; 
                        a.Y = 23;
                        a.Width = 50;
                        a.Height = 25;
                        a.Timelinetext = from.Hour.ToString() + ":00" ;
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = mid;
                        a.Type = (short)AssignmentType.text;
                        a.Y = 23;
                        a.Width = 50;
                        a.Height = 25;
                        a.Timelinetext = mid.Hour.ToString() + ":00";
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = late;
                        a.Type = (short)AssignmentType.text;
                        a.Y = 23;
                        a.Width = 50;
                        a.Height = 25;
                        a.Timelinetext = late.Hour.ToString() + ":00";
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = end;
                        a.Type = (short)AssignmentType.text;
                        a.Y = 23;
                        a.Width = 50;
                        a.Height = 25;
                        a.Timelinetext = end.Hour.ToString() + ":00";
                        TimelineData.Add(a);

                        a = new AssignmentViewModel(this);
                        a.From = from;
                        a.Type = (short)AssignmentType.text;
                        a.Y = 0;
                        a.Width = 150;
                        a.Height = 25;
                        a.Fontsize = 14;
                        a.Timelinetext = from.ToShortDateString();
                        TimelineData.Add(a);
                    }
                    if (j == 1)
                        d = d.AddDays(1);
                    else
                        d = d.AddDays(-1);
                }
            }
        }

        #region IScheduleCallback Members
        /// <summary>
        /// get the current user id
        /// </summary>
        /// <returns>windows user id</returns>
        public string GetUserId()
        {
            return WindowsIdentity.GetCurrent().Name;
        }

        /// <summary>
        /// the method is called by the server, when user information is updated
        /// </summary>
        /// <param name="usr">user id</param>
        /// <param name="name">user name</param>
        /// <param name="inactivity">inactivity in seconds</param>
        public void OnUserUpdated(string usr, string name, int inactivity)
        {
            bool found = false;
            foreach (UserViewModel u in UserList)
            {
                if (u.User == usr)
                {
                    found = true;
                    u.Name = name;
                    u.InactiveMinutes = inactivity;
                }
            }

            if (!found)
            {
                UserViewModel u = new UserViewModel(this);
                u.User = usr;
                u.Name = name;
                u.InactiveMinutes = inactivity;
                UserList.Add(u);
            }
        }

        /// <summary>
        /// is called by the server, when an user unsubscribes
        /// </summary>
        /// <param name="usr">user id</param>
        public void OnUserRemoved(string usr)
        {
            UserViewModel delobj = null;
            foreach (UserViewModel u in UserList)
            {
                if (u.User == usr)
                    delobj = u;

            }
            if (delobj != null)
                UserList.Remove(delobj);
        }

        /// <summary>
        /// is called by the server when the server returns the result of a customer query
        /// </summary>
        /// <param name="cust">collection of customers</param>
        public void OnGetCustomerFinished(ObservableCollection<CustIBaseView> cust)
        {
            ObservableCollection<CustIBaseViewModel> helplist = new ObservableCollection<CustIBaseViewModel>();
            IsBusy = false;
            AssignmentViewModel current = null;
            foreach (CustIBaseView c in cust)
            {
                CustIBaseViewModel mod = new CustIBaseViewModel(c);
                
                mod.Model.Assignments = c.Assignments;
                ObservableCollection<AssignmentViewModel> avm = new ObservableCollection<AssignmentViewModel>();
                var assignordered = from a in mod.Model.Assignments
                                    orderby a.FromDate descending
                                    select a;
                foreach (Assignment a in assignordered)
                {
                    if (a.Employee != null)
                    {
                        AssignmentViewModel av = new AssignmentViewModel(this);
                        av.Employee = Employeedict[a.Employee];
                        av.From = a.FromDate;
                        av.To = a.ToDate;
                        av.Type = a.Type;
                        av.Model.Id = a.Id;
                        av.Model.Ibase = (long)c.Ibase;
                        av.State = (AssignmentState)a.State;
                        av.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Assignment_PropertyChanged);
                        avm.Add(av);

                        if (_actualSelectedAssignment != null)
                        {
                            if (av.Model.Id == _actualSelectedAssignment.Model.Id)
                            {
                                current = av;
                            }
                        }
                    }
                }
                mod.AssignmentCollectionView = new ListCollectionView(avm);
                if (current != null)
                    mod.AssignmentCollectionView.MoveCurrentTo(current);
                
                mod.AssignmentCollectionView.CurrentChanged += new EventHandler(AssignmentCollectionView_CurrentChanged);                
                mod.AssignmentViewmodels = avm;
                mod.Model = c;
                /*
                mod.City = c.City;
                mod.HNr = c.HNr;
                mod.Ibase = c.Ibase;
                mod.IBaseCity = c.IBaseCity;
                mod.IBaseCityNr = c.IBaseCityNr;
                mod.IBaseHNr = c.IBaseHNr;
                mod.IBasePostCode = c.IBasePostCode;
                mod.IBaseStreet = c.IBaseStreet;
                mod.Installations = c.Installations;
                mod.lat = c.lat;
                mod.lon = c.lon;
                mod.Name = c.Name;
                mod.nextMaint = c.nextMaint;
                mod.Nr = c.Nr;
                mod.Post_code = c.Post_code;
                mod.Street = c.Street;
                mod.Phone = c.Phone;
                mod.Info = c.Info;
                 * */
                helplist.Add(mod);
            }
            Customers = helplist;
            CustomerCollectionView = new ListCollectionView(Customers);
            CustomerCollectionView.CurrentChanged += new EventHandler(CustomerCollectionView_CurrentChanged);
            if(current == null)
                if (helplist.Count >= 1)
                    selectAssignmentFromCurrentCustomer(helplist[0].Model);
            
        }

        /// <summary>
        /// is called when the current item of the assignment collection view is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AssignmentCollectionView_CurrentChanged(object sender, EventArgs e)
        {
            CollectionView c = (CollectionView)sender;
            
            if(c.CurrentItem != null)
            {
                if (c.CurrentItem.GetType().ToString() == "APlan.ViewModel.AssignmentViewModel")
                {
                    AssignmentViewModel av = (AssignmentViewModel) c.CurrentItem;
                    bool panToAssignment = false;
                    if (_actualSelectedAssignment != null && av.Model != null)
                    {
                        if (av.Model.Id != _actualSelectedAssignment.Model.Id)
                        {
                            panToAssignment = true;
                        }
                    }
                    else
                        panToAssignment = true;
                    if (panToAssignment)
                    {   
                        selectAssignment(av.GetAssignmentObject());
                    }
                }
            }
        }

        /// <summary>
        /// is called by the server when a assignment query is finished
        /// </summary>
        /// <param name="assignments"></param>
        public void OnGetAssignmentsFinished(ObservableCollection<AssignmentIBaseView> assignments)
        {            
            ViewModelLocator.MainStatic.Assignments.Clear();
            ViewModelLocator.MainStatic.Assignmentdict.Clear();
            generateBackgroundLines();
            foreach (AssignmentIBaseView a in assignments)
            {
                AssignmentViewModel av = new AssignmentViewModel(this,a);
                
                //av.IBase1 = ibases[(long)a.ibase];
                av.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Assignment_PropertyChanged);
                
                ViewModelLocator.MainStatic.Assignments.Add(av);
                ViewModelLocator.MainStatic.Assignmentdict.Add(av.Model.Id,av);
            }
        }

        /// <summary>
        /// is called by the server on undo or redo
        /// </summary>
        /// <param name="a">assignmentundoredo object</param>
        public void OnUndoRedo(AssignmentUndoRedo a)
        {
            /* The variable undohelp is used to avoid updates on database when doing undo/redo. 
             * In Assignment_PropertyChanged no changes are made when _undohelp is true */
            _undohelp = true;
            if (a.Action == (int)AssignmentAction.Update)
            {
                AssignmentViewModel av = ViewModelLocator.MainStatic.Assignmentdict[a.Id];
                av.Employee = ViewModelLocator.MainStatic.Employeedict[a.Emplyoee];
                av.Fix = a.Fix;
                av.From = a.FromDate;
                av.Model.Ibase = (long)a.IBase;
                av.State = (AssignmentState)a.State;
                av.To = a.ToDate;
                av.Type = a.Type;
                av.SetToNextEmployee();
            }
            else
            {
                if ((a.Undo && a.Action == (int)AssignmentAction.Insert) || (!a.Undo && a.Action == (int)AssignmentAction.Delete))
                {
                    // remove the assignment
                    AssignmentViewModel av = ViewModelLocator.MainStatic.Assignmentdict[a.Id];
                    executeRemoveAssignmentCommand(av);
                }
                else
                {

                }
            }
            _undohelp = false;
        }

        /// <summary>
        /// is called by the server, when an user drag an assignment
        /// </summary>
        /// <param name="a">updated assignment</param>
        public void OnAssignmentUpdate(AssignmentDistr a)
        {
            // Assignments with Employee == null are open failures
            
                if (a.DBAssignment.Employee != null)
                {
                    AssignmentViewModel av, custav = null;
                    _distributedAssignment = true;
                    if (ViewModelLocator.MainStatic.Assignmentdict.ContainsKey(a.DBAssignment.Id))
                    {
                        av = ViewModelLocator.MainStatic.Assignmentdict[a.DBAssignment.Id];
                        av.Y = a.Y;
                        av.Opacity = a.Opacity;
                        foreach (CustIBaseViewModel c in this.Customers)
                        {
                            if (c.Model.Ibase == av.Model.Ibase)
                            {
                                
                                foreach (AssignmentViewModel cav in c.AssignmentViewmodels)
                                {
                                    if (cav.Model.Id == av.Model.Id)
                                    {
                                        custav = cav;                                       
                                    }
                                }
                                
                            }
                        }

                    }
                    else
                    {
                        //new assignment inserted
                        av = new AssignmentViewModel(this, a.AssignIBaseView);
                        av.Model.Id = a.DBAssignment.Id;
                        av.Model.Ibase = (long)a.DBAssignment.IBase;

                        //handle updates on the assignment
                        av.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Assignment_PropertyChanged);

                        // insert the assignment into the assignment collections
                        ViewModelLocator.MainStatic.Assignmentdict.Add(a.DBAssignment.Id, av);
                        ViewModelLocator.MainStatic.Assignments.Add(av);
                        foreach (CustIBaseViewModel c in this.Customers)
                        {
                            if (c.Model.Ibase == av.Model.Ibase)
                            {
                                c.AssignmentViewmodels.Add(av);
                            }
                        }
                    }
                    av.From = (DateTime)a.DBAssignment.FromDate;
                    av.To = (DateTime)a.DBAssignment.ToDate;
                    av.Model.fix = a.DBAssignment.fix;

                    av.Employee = ViewModelLocator.MainStatic.Employeedict[a.DBAssignment.Employee];
                    if (av.Y == 0)
                    {
                        av.SetToNextEmployee();
                    }
                    
                    if (custav != null)
                    {
                        custav.From = av.From;
                        custav.To = av.To;
                        custav.Employee = av.Employee;
                        custav.Model.fix = av.Model.fix;
                    }
                    _distributedAssignment = false;
                    /*
                    Assignments.Remove(av);
                    Assignments.Add(av);
                     * */
                    //todo: optimize this!
                }
            
        }

        /// <summary>
        /// is called by the server, when an employee query is finished
        /// </summary>
        /// <param name="employees"></param>
        /// <param name="regiogroups"></param>
        public void OnGetEmployeesFinished(ObservableCollection<Employee> employees, Regiogroup[] regiogroups)
        {            
            int posnr = 1;
            foreach (Employee e in employees)
            {
                if (e.technician)
                {
                    EmployeeViewmodel ev = new EmployeeViewmodel(this);
                    ev.Name = e.name;
                    ev.User = e.User;
                    ev.City = e.Cities;
                    
                    ev.PosNr = posnr;
                    ev.Height = this.ElementHeight;
                    ev.Technician = e.technician;
                    ViewModelLocator.MainStatic.Employees.Add(ev);
                    ViewModelLocator.MainStatic.Employeedict.Add(ev.User, ev);
                    posnr++;
                }                
            }
        }

        /// <summary>
        /// is called by the server, when an new assignment has been inserted
        /// </summary>
        /// <param name="a"></param>
        /// <param name="panToAssignment"></param>
        public void OnCreateAssignmentCallback(AssignmentIBaseView a, Boolean panToAssignment)
        {
            if (!Assignmentdict.ContainsKey(a.Id))
            {
                AssignmentViewModel av = new AssignmentViewModel(this, a);
                if (panToAssignment)
                {
                    if (_actualSelectedAssignment != null)
                    {
                        _actualSelectedAssignment.IsSelected = false;
                    }
                    av.IsSelected = true;
                    _actualSelectedAssignment = av;
                }
                av.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Assignment_PropertyChanged);
                
                Assignments.Add(av);
                Assignmentdict.Add(av.Model.Id, av);

                if (panToAssignment)
                {
                    GalaSoft.MvvmLight.Messaging.Messenger.Default.Send<AssignmentViewModel>(av);
                }
                //in the current list of customers the assignment has to be inserted too
                //(the information is not updated from the db)

                foreach (CustIBaseViewModel c in this.Customers)
                {
                    if (c.Model.Ibase == a.Ibase)
                    {                        
                        c.AssignmentViewmodels.Add(av);
                    }
                }

                RoutePlanProposed = true;
            }
        }

        /// <summary>
        /// is called by the server, when the routeplan is finished
        /// </summary>
        public void OnRouteplanFinished()
        {
            IsBusy = false;
            _routePlanProposed = true;
            CommandManager.InvalidateRequerySuggested();
        }

        /// <summary>
        /// is called by the server, when a new open failure has been inserted
        /// </summary>
        /// <param name="openfailures"></param>
        public void OnOpenFailureUpdate(ObservableCollection<CustIBaseView> openfailures)
        {
            ObservableCollection<CustIBaseViewModel> helplist = new ObservableCollection<CustIBaseViewModel>();
            foreach (CustIBaseView c in openfailures)
            {
                CustIBaseViewModel mod = new CustIBaseViewModel(c);
                /*
                mod.Assignments = c.Assignments;
                mod.City = c.City;
                mod.HNr = c.HNr;
                mod.Ibase = c.Ibase;
                mod.IBaseCity = c.IBaseCity;
                mod.IBaseCityNr = c.IBaseCityNr;
                mod.IBaseHNr = c.IBaseHNr;
                mod.IBasePostCode = c.IBasePostCode;
                mod.IBaseStreet = c.IBaseStreet;
                mod.Installations = c.Installations;
                mod.lat = c.lat;
                mod.lon = c.lon;
                mod.Name = c.Name;
                mod.nextMaint = c.nextMaint;
                mod.Nr = c.Nr;
                mod.Post_code = c.Post_code;
                mod.Street = c.Street;
                mod.Phone = c.Phone;
                mod.Info = c.Info;
                 * */
                helplist.Add(mod);
            }
            OpenFailures = helplist;
            OpenFailureCollectionView = new ListCollectionView(OpenFailures);
        }

        /// <summary>
        /// is called by the server, when the heatmap is updated
        /// </summary>
        /// <param name="hm"></param>
        public void OnHeatmapUpdate(HeatMapViewModel[] hm)
        {
            OpenCustHeatMap = new ObservableCollection<HeatMapViewModel>( hm);
        }

        /// <summary>
        /// is called by the server, when a message is updated
        /// </summary>
        /// <param name="msg"></param>
        public void OnMessageUpdate(Message[] msg)
        {
            List<MessageViewModel> l = new List<MessageViewModel>();
            foreach (Message m in msg)
            {
                MessageViewModel mv = new MessageViewModel();
                mv.Message = m.Messagetext;
                mv.PostingDate = m.PostingDate;
                mv.ValidTo = m.ValidTo;
                mv.ID = m.id;
                mv.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(message_PropertyChanged);
                l.Add(mv);
            }
            Messages = new ObservableCollection<MessageViewModel>(l.ToArray());
            Messages.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Messages_CollectionChanged);
        }

        void Messages_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                MessageViewModel mv = (MessageViewModel)e.NewItems[0];
                mv.PostingDate = DateTime.Now;
                mv.ValidTo = DateTime.Now.AddDays(7);
                mv.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(message_PropertyChanged);
            }
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
            {
                MessageViewModel mv = (MessageViewModel)e.OldItems[0];
                Message m = new Message();
                m.id = mv.ID;
                proxy.DeleteMessage(m);
            }

        }
        /// <summary>
        /// is called, when a property of a message is updated
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void message_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            MessageViewModel mv = (MessageViewModel)sender;
            Message m = new Message();
            m.Messagetext = mv.Message;
            m.PostingDate = mv.PostingDate;
            m.ValidTo = mv.ValidTo;
            m.id = mv.ID;

            if (mv.ID == Guid.Empty)
            {
                m.id = Guid.NewGuid();
                proxy.InsertMessage(m);
            }
            else
                proxy.UpdateMessage(m);
        }

        /// <summary>
        /// is called by the server, when the availability of undo or redo changes
        /// </summary>
        /// <param name="undo"></param>
        /// <param name="redo"></param>
        public void UndoRedoAvailable(Boolean undo, Boolean redo)
        {
            _undopossible = undo;
            _redopossible = redo;
            CommandManager.InvalidateRequerySuggested();
        }
        #endregion

        /// <summary>
        /// is called when an property of an assignment is updated
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Assignment_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            // when undo or a distribution of an assignment is the reason for the call: do nothing
            if (!_undohelp && !_distributedAssignment)
            {
                // when the following properties are updated -> distribute the changes
                if (e.PropertyName == "IsDragging" || e.PropertyName == "Fix" || e.PropertyName == "X" || e.PropertyName =="Y")
                {
                    AssignmentViewModel assignment = (AssignmentViewModel)sender;
                    AssignmentDistr distrAss = new AssignmentDistr();
                    distrAss.DBAssignment = assignment.GetAssignmentObject();
                    distrAss.Y = assignment.Y;
                    distrAss.Opacity = assignment.Opacity;
                    proxy.DistributeAssignment(distrAss, WindowsIdentity.GetCurrent().Name);
                    if (assignment.IsDragging)
                    {
                    }
                    else
                    {
                        //assignment.State = AssignmentState.open;
                        proxy.DistributeAssignment(distrAss, "");
                        proxy.UpdateAssignment(assignment.GetAssignmentObject());
                    }
                }
                // select the assignment on plan and detail, when isSelected is updated
                if (e.PropertyName == "IsSelected")
                {
                    bool found = false;
                    AssignmentViewModel assignment = (AssignmentViewModel)sender;
                    if (assignment.IsSelected)
                    {
                        CustIBaseViewModel current = (CustIBaseViewModel)CustomerCollectionView.CurrentItem;
                        if (current != null)
                        {
                            if (assignment.Model.Ibase != current.Model.Ibase)
                            {
                                foreach (CustIBaseViewModel c in Customers)
                                {
                                    if (assignment.Model.Ibase == c.Model.Ibase)
                                    {
                                        found = true;
                                        CustomerCollectionView.MoveCurrentTo(c);

                                    }
                                }
                            }
                            foreach (AssignmentViewModel cav in current.AssignmentViewmodels)
                            {
                                if (cav.Model.Id == assignment.Model.Id)
                                {
                                    current.AssignmentCollectionView.MoveCurrentTo(cav);
                                }
                            }
                        }
                        if (!found)
                        {
                            if (current == null)
                                proxy.GetCustomerByIBase(assignment.Model.Ibase);
                            else if (current.Model.Ibase != assignment.Model.Ibase)
                                proxy.GetCustomerByIBase(assignment.Model.Ibase);
                        }
                    }

                }
                // distribute the assignment, when the state or the time has been edited manualy
                if (e.PropertyName == "State" || e.PropertyName == "ManualTimeChange")
                {
                    AssignmentViewModel help = (AssignmentViewModel)sender;
                    AssignmentViewModel av = Assignmentdict[help.Model.Id];
                    _distributedAssignment = true;
                    av.State = help.State;
                    av.From = help.From;
                    av.To = help.To;
                    
                    proxy.UpdateAssignment(av.GetAssignmentObject());
                    _distributedAssignment = false;
                }
            }
            
        }

        
    }
}