﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Collections.ObjectModel;
using APlan.ViewModel;
namespace APlan.Service
{
    public enum AssignmentState
    {
        open = 1, proposal = 2, inform = 3, finished =4
    }

    public enum AssignmentType
    {
        maintenance = 1, failure = 2, help = 3, text = 4, bar = 5
    }
        
    [ServiceContract(CallbackContract = typeof(IAPlanCallback))]        
    public interface IAPlanService
    {

        [OperationContract(IsOneWay=true)]
        void Subscribe(string usr);

        [OperationContract(IsOneWay=true)]
        void Unsubscribe(string usr);

        [OperationContract]
        Employee[] GetUserList();

        [OperationContract(IsOneWay=true)]
        void GetCustomer(string text);

        [OperationContract(IsOneWay = true)]
        void GetCustomerByIBase(long ibase);

        [OperationContract(IsOneWay = true)]
        void UpdateIBase(CustIBaseView data);

        [OperationContract(IsOneWay = true)]
        void GetAssignments(DateTime from, DateTime to);

        [OperationContract(IsOneWay = true)]
        void DistributeAssignment(AssignmentDistr a, string originUser);

        [OperationContract(IsOneWay = true)]
        void UpdateAssignment(Assignment aInp);

        [OperationContract(IsOneWay = true)]
        void InsertAssignment(Assignment a);

        [OperationContract(IsOneWay = true)]
        void CreateAssignment(CustIBaseView cust, DateTime minDate, DateTime maxDate, TimeSpan timeFrom, TimeSpan timeTo);

        [OperationContract(IsOneWay = true)]
        void CreateRoutePlan(HeatMapViewModel[] cust, DateTime StartDate, DateTime EndDate, TimeSpan beginTime, TimeSpan endTime);

        [OperationContract(IsOneWay = true)]
        void AcknowledgeRoutePlan(Boolean ack);
        
        [OperationContract(IsOneWay = true)]
        void RemoveAssignment(Assignment a);

        [OperationContract(IsOneWay = true)]
        void GetEmployees();

        [OperationContract(IsOneWay = true)]
        void CreateOpenFailure(Int64 ibase);

        [OperationContract(IsOneWay = true)]
        void GetOpenFailures();

        [OperationContract(IsOneWay = true)]
        void RemoveOpenFailure(Int64 ibase);

        [OperationContract(IsOneWay = true)]
        void UpdateMessage(Message msg);

        [OperationContract(IsOneWay = true)]
        void InsertMessage(Message msg);

        [OperationContract(IsOneWay = true)]
        void DeleteMessage(Message msg);

        [OperationContract(IsOneWay = true)]
        void UndoRedo(Boolean undo);

        [OperationContract]
        HeatMapViewModel[] GetOpenCustomers();

        [OperationContract]
        HeatMapViewModel GetOpenCustomersDetail(HeatMapViewModel h);

        [OperationContract]
        City GetCity(long citynr);

        [OperationContract]
        void Test();
    }

    public interface IAPlanCallback
    {
        [OperationContract(IsOneWay=true)]
        void OnUserUpdated(string usr, string name, int inactiveMin);

        [OperationContract(IsOneWay=true)]
        void OnUserRemoved(string usr);

        [OperationContract(IsOneWay = true)]
        void OnGetCustomerFinished(ObservableCollection<CustIBaseView> cust);

        [OperationContract /*(IsOneWay = true)*/]
        void OnGetAssignmentsFinished(ObservableCollection<AssignmentIBaseView> assignments);

        [OperationContract(IsOneWay = true)]
        void OnGetEmployeesFinished(ObservableCollection<Employee> employees, Regiogroup[] regiogroups);        

        [OperationContract(IsOneWay = true)]
        void OnAssignmentUpdate(AssignmentDistr a);
        
        [OperationContract(IsOneWay = true)]
        void OnCreateAssignmentCallback(AssignmentIBaseView a, Boolean panToAssignemnt);

        [OperationContract(IsOneWay = true)]
        void OnOpenFailureUpdate(ObservableCollection<CustIBaseView> failures);

        [OperationContract(IsOneWay = true)]
        void OnHeatmapUpdate(HeatMapViewModel[] hm);

        [OperationContract(IsOneWay = true)]
        void OnRouteplanFinished();

        [OperationContract(IsOneWay = true)]
        void OnMessageUpdate(Message[] msg);

        [OperationContract(IsOneWay = true)]
        void OnUndoRedo(AssignmentUndoRedo a);

        [OperationContract(IsOneWay = true)]
        void UndoRedoAvailable(Boolean undo, Boolean redo);
        [OperationContract]
        string GetUserId();
    }
}
