﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace APlan.Service
{
    /// <summary>
    /// A route are all assignments of an technician per day
    /// </summary>
    public class Route
    {
        public DateTime Date;
        public Employee Technician;
        public City LastDestination;
        public List<Assignment> Assignments;

        private Routeplan routeplan;

        public Route(Routeplan r)
        {
            routeplan = r;
        }

        public void InsertAssignments(List<Assignment> ins)
        {
            DataClassesDataContext context = new DataClassesDataContext();
            foreach (Assignment a in ins)
            {
                var assignupd = ( from adb in context.Assignments
                                where adb.Id == a.Id
                                select adb ).Single();
                if (Assignments.Count != 0)
                    assignupd.FromDate = Assignments[Assignments.Count() - 1].ToDate;
                else
                    assignupd.FromDate = new DateTime(Date.Year,Date.Month,Date.Day,routeplan.FromTime.Hours,routeplan.FromTime.Minutes,0);
                assignupd.ToDate = assignupd.FromDate.AddMinutes(routeplan.AssignmentDuration);
                assignupd.State = (int) AssignmentState.proposal;
                assignupd.Employee = Technician.User;
                Assignments.Add(assignupd);
                
            }
            context.SubmitChanges();
        }

        public double DistFromLastHome
        {
            get
            {
                double ret = 0;
                if (Assignments.Count() != 0)
                {
                    City LastCity = Assignments[Assignments.Count() - 1].IBase1.City1;
                    //euclidian distance to the home of the technician
                    ret = Math.Pow((LastCity.lat - Technician.City1.lat), 2)
                         + Math.Pow((LastCity.lon - Technician.City1.lon), 2);
                    ret = Math.Sqrt(ret);
                }
                return ret;
            }
        }

        public double DistFromHomeToFirst
        {
            get
            {
                double ret = 0;
                if (Assignments.Count() != 0)
                {
                    City FirstCity = routeplan.Cities[Assignments[0].IBase1.City];
                    //euclidian distance to the home of the technician
                    City TechnicianCity = routeplan.Cities[Technician.city];
                    ret = Math.Pow((FirstCity.lat - TechnicianCity.lat), 2)
                         + Math.Pow((FirstCity.lon - TechnicianCity.lon), 2);
                    ret = Math.Sqrt(ret);
                }
                return ret;
            }

        }

        

        public Assignment CreateAssignmentAtEnd(CustIBaseView cust, AssignmentState state)
        {
            DataClassesDataContext context = new DataClassesDataContext();
            Assignment a = new Assignment();
            if (Assignments.Count != 0)
            {
                a.FromDate = Assignments[Assignments.Count() - 1].ToDate;
            }
            else
                a.FromDate = new DateTime(Date.Year, Date.Month, Date.Day,routeplan.FromTime.Hours, routeplan.FromTime.Minutes,0);
            
            a.ToDate = a.FromDate.AddMinutes(routeplan.AssignmentDuration);
            a.Employee = Technician.User;
            //a.Employee1 = Technician;
            a.fix = false;
            a.IBase = cust.Ibase;
            //a.IBase1 = cust.IBase1;
            if (context.Assignments.Count() != 0)
                a.Id = (from maxass in context.Assignments select maxass.Id).Max() + 1;
            else
                a.Id = 1;
            a.Type = (short)AssignmentType.maintenance;
            a.State = (short)state;
            Assignments.Add(a);
            
            context.Assignments.InsertOnSubmit(a);
            context.SubmitChanges();

            return a;
        }

        public double GetDistBetweenLastAndCity(City city)
        {
            double ret = 0;
            City routeCity;
            if (Assignments.Count() != 0)
            {
                routeCity = Assignments[Assignments.Count() - 1].IBase1.City1;
            }
            else
            {
                routeCity = Technician.City1;
            }
            ret = Math.Pow((routeCity.lat - city.lat), 2)
                         + Math.Pow((routeCity.lon - city.lon), 2);
            ret = Math.Sqrt(ret);
            return ret;
        }
    }
}
