﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace APlan.Service
{
    public class Routeplan
    {
        private System.Collections.Generic.List<Route> routes;
        private Double[,] savings;
        public Dictionary<int, City> Cities;
        //the duration in minutes for every new assignment
        public int AssignmentDuration;
        public int RoutesPerDay;
        public DateTime FromDate, EndDate;
        public TimeSpan FromTime, EndTime;
        public Routeplan(DateTime fromDate, DateTime endDate, TimeSpan fromTime, TimeSpan endTime)
        {
            DataClassesDataContext context = new DataClassesDataContext();
            Cities = new Dictionary<int, City>();
            foreach (City c in context.Cities)
            {
                Cities.Add(c.ID, c);
            }
            AssignmentDuration = 90;
            RoutesPerDay = 5;
            FromDate = fromDate;
            EndDate = endDate;

            FromTime = fromTime;
            EndTime = endTime;

            routes = new List<Route>();
        }

        public Routeplan(int assignmentduration, int routesperday,
                 DateTime fromdate, DateTime enddate, TimeSpan fromtime, TimeSpan endtime) : this(fromdate, enddate, fromtime, endtime)
        {
            AssignmentDuration = assignmentduration;
            RoutesPerDay = routesperday;
        }

        public void GenerateRoutes(IQueryable<Assignment> assignments)
        {
            DataClassesDataContext context = new DataClassesDataContext();
            var employees = from e in context.Employees 
                            where e.technician == true select e;
            foreach (Employee e in employees)
            {
                DateTime date = FromDate;
                while (date <= EndDate)
                {
                    if (date.DayOfWeek != DayOfWeek.Saturday && date.DayOfWeek != DayOfWeek.Sunday)
                    {
                        DateTime low = new DateTime(date.Year, date.Month, date.Day, 0, 0, 0);
                        DateTime high = new DateTime(date.Year, date.Month, date.Day, 23, 59, 59);
                        Route r = new Route(this);
                        r.Date = date;
                        r.Technician = e;

                        var routeAssignments = from a in assignments
                                               where a.Employee == e.User
                                                 && a.ToDate > low && a.ToDate < high
                                               orderby a.ToDate
                                               select a;
                        if (routeAssignments.Count() == 0)
                        {
                            r.LastDestination = e.City1;
                        }
                        else
                        {
                            DateTime maxDate = DateTime.MinValue;
                            foreach (Assignment assign in routeAssignments)
                            {
                                if (assign.ToDate > maxDate)
                                {
                                    r.LastDestination = assign.IBase1.City1;
                                }
                            }
                            
                        }
                        r.Assignments = new List<Assignment>(routeAssignments.ToArray());

                        routes.Add(r);
                    }
                    date = date.AddDays(1);
                }
            }
        }

        private double calculateDistance(City c1, City c2)
        {
            double ret = 0;
            
            ret = Math.Pow((c1.lat - c2.lat), 2)
                         + Math.Pow((c1.lon - c2.lon), 2);
            ret = Math.Sqrt(ret);
            return ret;
        }

        public void CreatePendingtours(List<CustIBaseView> openCust, DateTime startdate)
        {
            DataClassesDataContext context = new DataClassesDataContext();
            

            //find the next Employees
            foreach (CustIBaseView c in openCust)
            {
                double mindist = Double.MaxValue;
                DateTime minDateEmployee = DateTime.MaxValue;
                Route minRoute = null;
                foreach (Route r in routes)
                {
                    if (r.Assignments.Count == 0)
                    {
                        double dist = r.GetDistBetweenLastAndCity(c.IBase1.City1);
                        if (dist <= mindist)
                        {
                            
                            if (dist == mindist)
                            {
                                //special case: take the technician that is free earlier
                                if (r.Date < minRoute.Date)
                                {
                                    minRoute = r;                                    
                                }
                            }
                            else
                            {
                                mindist = dist;
                                minRoute = r;
                            }
                        }
                    }
                }
                Assignment a = minRoute.CreateAssignmentAtEnd(c, AssignmentState.proposal);
                
            }
        }

        public void CalculateSavings()
        {
            savings = new Double[routes.Count,routes.Count];
            Boolean[,] finished = new Boolean[routes.Count, routes.Count];
            Boolean savingFound = true;
            Double maxsaving = Double.MinValue;
            int maxi = - 1, maxj = -1;
            for (int i = 0; i < routes.Count; i++)
                for (int j = 0; j < routes.Count; j++)
                    finished[i, j] = false;

            //calculate savings matrix
            maxsaving = Double.MinValue;
            for (int i = 0; i < routes.Count; i++)
            {
                for (int j = 0; j < routes.Count; j++)
                {
                    if (i == j || finished[i, j])
                        savings[i, j] = Double.MinValue;
                    else
                    {
                        Route r1 = routes[i];
                        Route r2 = routes[j];
                        City lastcity = Cities[r2.Assignments[r2.Assignments.Count - 1].CustIBaseView.IBase1.City];
                        savings[i, j] = r1.DistFromLastHome + r2.DistFromHomeToFirst - r1.GetDistBetweenLastAndCity(lastcity);
                        if (savings[i, j] > maxsaving && savings[i, j] >= 0)
                        {
                            savingFound = true;
                            maxsaving = savings[i, j];
                            maxi = i;
                            maxj = j;
                        }
                    }
                }
            }

            while (savingFound)
            {                
                if (maxi != -1)
                {
                    Route r1, r2;
                    int early, later;
                    if (routes[maxi].Date > routes[maxj].Date)
                    {
                        early = maxj;
                        later = maxi;                        
                    }
                    else
                    {
                        early = maxi;
                        later = maxj;                        
                    }
                    r1 = routes[early];
                    r2 = routes[later];
                    if (r1.Assignments.Count + r2.Assignments.Count <= RoutesPerDay)
                    {
                        r1.InsertAssignments(r2.Assignments);
                        r2.Assignments = new List<Assignment>();
                        for (int i = 0; i < routes.Count; i++)
                        {
                            finished[i, later] = true;
                            finished[later, i] = true;
                        }
                    }
                    finished[maxi, maxj] = true;
                }
                savingFound = false;
                maxi = -1; maxj = -1;
                maxsaving = Double.MinValue;
                for (int i = 0; i < routes.Count; i++)
                {
                    for (int j = 0; j < routes.Count; j++)
                    {
                        if (i == j || finished[i, j])
                            savings[i, j] = Double.MinValue;
                        else
                        {                            
                            if (savings[i, j] > maxsaving && savings[i, j] >= 0)
                            {
                                savingFound = true;
                                maxsaving = savings[i, j];
                                maxi = i;
                                maxj = j;
                            }
                        }
                    }
                }
                
            }
        }

        public void CalculateSavings2()
        {
            savings = new Double[routes.Count, routes.Count];
            Boolean[,] finished = new Boolean[routes.Count, routes.Count];
            Boolean savingFound = true;
            Double maxsaving = Double.MinValue;
            int maxi = -1, maxj = -1;
            for (int i = 0; i < routes.Count; i++)
                for (int j = 0; j < routes.Count; j++)
                    finished[i, j] = false;

            while (savingFound)
            {
                savingFound = false;
                maxsaving = Double.MinValue;
                for (int i = 0; i < routes.Count; i++)
                {
                    for (int j = 0; j < routes.Count; j++)
                    {
                        if (i == j || finished[i, j])
                            savings[i, j] = Double.MinValue;
                        else
                        {
                            Route r1 = routes[i];
                            Route r2 = routes[j];
                            City lastcity = Cities[r2.Assignments[r2.Assignments.Count - 1].CustIBaseView.IBase1.City];
                            savings[i, j] = r1.DistFromLastHome + r2.DistFromHomeToFirst - r1.GetDistBetweenLastAndCity(lastcity);
                            if (savings[i, j] > maxsaving && savings[i, j] >= 0)
                            {
                                savingFound = true;
                                maxsaving = savings[i, j];
                                maxi = i;
                                maxj = j;
                            }
                        }
                    }
                }

                if (maxi != -1)
                {
                    Route r1, r2;
                    int early, later;
                    if (routes[maxi].Date > routes[maxj].Date)
                    {
                        early = maxj;
                        later = maxi;
                    }
                    else
                    {
                        early = maxi;
                        later = maxj;
                    }
                    r1 = routes[early];
                    r2 = routes[later];
                    if (r1.Assignments.Count + r2.Assignments.Count <= RoutesPerDay)
                    {
                        r1.InsertAssignments(r2.Assignments);
                        r2.Assignments = new List<Assignment>();
                        for (int i = 0; i < routes.Count; i++)
                        {
                            finished[i, later] = true;
                            finished[later, i] = true;
                        }
                    }
                    finished[maxi, maxj] = true;
                }

            }
        }

        public void FillGaps()
        {
            Boolean found = true;
            while (found)
            {
                found = false;
                foreach (Route route in routes)
                {
                    if (route.Assignments.Count != 0)
                    {
                        DateTime mindate = DateTime.MaxValue;
                        Route mindateRoute = null;
                        foreach (Route r in routes)
                        {
                            if (route.Technician == r.Technician && r.Assignments.Count == 0 && route.Date > r.Date)
                            {
                                mindate = r.Date;
                                mindateRoute = r;
                                found = true;
                            }
                        }

                        if (route.Date > mindate)
                        {
                            mindateRoute.InsertAssignments(route.Assignments);
                            route.Assignments = new List<Assignment>();
                        }
                    }

                }
            }
        }
        public void DeleteEmptyRoutes()
        {
            List<Route> rnew = new List<Route>();
            foreach (Route r in routes)
            {
                if (r.Assignments.Count != 0)
                    rnew.Add(r);
            }
            routes = rnew;
        }

        public AssignmentDistr[] GetAllAssignments()
        {
            List<AssignmentDistr> l = new List<AssignmentDistr>();
            foreach (Route r in routes)
            {
                foreach (Assignment a in r.Assignments)
                {
                    AssignmentDistr ad = new AssignmentDistr();
                    ad.DBAssignment = a;
                    l.Add(ad);
                }
            }
            return l.ToArray();
        }

        public Assignment CreateAssignment(CustIBaseView cust)
        {
            Assignment ret = new Assignment();
            Route targetRoute = null;
            double minDist = double.MaxValue;

            DataClassesDataContext context = new DataClassesDataContext();

            var customerdb = (from cu in context.CustIBaseViews
                              where cu.Ibase == cust.Ibase
                              select cu).Single();

            int maxassignment = 0;
            DateTime actdate = DateTime.MaxValue;
            foreach (Route r in routes)
            {
                
                City custCity = customerdb.IBase1.City1;
                double dist = r.GetDistBetweenLastAndCity(custCity);
                if (r.Assignments.Count() < RoutesPerDay)
                {
                    if (dist <= minDist)
                    {
                        if (dist == minDist)
                        {
                            // when the distance is equal, add it to a route with more assignments
                            if (r.Assignments.Count() > maxassignment)
                            {
                                maxassignment = r.Assignments.Count();
                                targetRoute = r;
                                actdate = r.Date;
                                minDist = dist;
                            }
                            else
                            {
                                if (r.Date < actdate)
                                {
                                    maxassignment = r.Assignments.Count();
                                    targetRoute = r;
                                    actdate = r.Date;
                                    minDist = dist;
                                }
                            }
                        }
                        else
                        {
                            targetRoute = r;
                            maxassignment = r.Assignments.Count();
                            minDist = dist;
                        }
                    }
                }
            }
            ret = targetRoute.CreateAssignmentAtEnd(customerdb, AssignmentState.proposal);
            
            return ret;
        }
    }
}
